@@ -17,7 +17,7 @@ my(@XSStack);	# Stack of conditionals and INCLUDEs
 my($XSS_work_idx, $cpp_next_tmp);
 
 use vars qw($VERSION);
-$VERSION = '2.07';
+$VERSION = '2.10';
 
 use vars qw(%input_expr %output_expr $ProtoUsed @InitFileCode $FH $proto_re $Overload $errors $Fallback
 	    $cplusplus $hiertype $WantPrototypes $WantVersionChk $except $WantLineNumbers
@@ -54,6 +54,7 @@ sub process_file {
 	   argtypes => 1,
 	   typemap => [],
 	   output => \*STDOUT,
+	   csuffix => '.c',
 	   %args,
 	  );
 
@@ -95,6 +96,7 @@ sub process_file {
     die "Missing required parameter 'filename'" unless $_;
     $filepathname = $_;
     ($dir, $filename) = (dirname($_), basename($_));
+    $filepathname =~ s/\\/\\\\/g;
     $IncludedFiles{$_}++;
   }
   
@@ -116,14 +118,15 @@ sub process_file {
   
   chdir($dir);
   my $pwd = cwd();
+  my $csuffix = $args{csuffix};
   
   if ($WantLineNumbers) {
     my $cfile;
     if ( $args{outfile} ) {
       $cfile = $args{outfile};
     } else {
-      $cfile = $filepathname;
-      $cfile =~ s/\.xs$/.c/i or $cfile .= ".c";
+      $cfile = $args{filename};
+      $cfile =~ s/\.xs$/$csuffix/i or $cfile .= $csuffix;
     }
     tie(*PSEUDO_STDOUT, 'ExtUtils::ParseXS::CountLines', $cfile, $args{output});
     select PSEUDO_STDOUT;
@@ -258,7 +261,20 @@ EOM
       my $podstartline = $.;
       do {
 	if (/^=cut\s*$/) {
-	  print("/* Skipped embedded POD. */\n");
+	  # We can't just write out a /* */ comment, as our embedded
+	  # POD might itself be in a comment. We can't put a /**/
+	  # comment inside #if 0, as the C standard says that the source
+	  # file is decomposed into preprocessing characters in the stage
+	  # before preprocessing commands are executed.
+	  # I don't want to leave the text as barewords, because the spec
+	  # isn't clear whether macros are expanded before or after
+	  # preprocessing commands are executed, and someone pathological
+	  # may just have defined one of the 3 words as a macro that does
+	  # something strange. Multiline strings are illegal in C, so
+	  # the "" we write must be a string literal. And they aren't
+	  # concatenated until 2 steps later, so we are safe.
+	  #     - Nicholas Clark
+	  print("#if 0\n  \"Skipped embedded POD.\"\n#endif\n");
 	  printf("#line %d \"$filepathname\"\n", $. + 1)
 	    if $WantLineNumbers;
 	  next firstmodule
@@ -276,7 +292,10 @@ EOM
     
     print $_;
   }
-  die "Didn't find a 'MODULE ... PACKAGE ... PREFIX' line\n" unless defined $_;
+  unless (defined $_) {
+    warn "Didn't find a 'MODULE ... PACKAGE ... PREFIX' line\n";
+    exit 0; # Not a fatal error for the caller process
+  }
 
     print <<"EOF";
 #ifndef PERL_UNUSED_VAR
@@ -285,7 +304,7 @@ EOM
 
 EOF
 
-  print "$ExtUtils::ParseXS::CountLines::SECTION_END_MARKER\n" if $WantLineNumbers;
+  print 'ExtUtils::ParseXS::CountLines'->end_marker, "\n" if $WantLineNumbers;
 
   $lastline    = $_;
   $lastline_no = $.;
@@ -340,7 +359,7 @@ EOF
 	   ." followed by a statement on column one?)")
       if $line[0] =~ /^\s/;
     
-    my ($class, $static, $elipsis, $wantRETVAL, $RETVAL_no_return);
+    my ($class, $externC, $static, $elipsis, $wantRETVAL, $RETVAL_no_return);
     my (@fake_INPUT_pre);	# For length(s) generated variables
     my (@fake_INPUT);
     
@@ -394,7 +413,8 @@ EOF
     blurt ("Error: Function definition too short '$ret_type'"), next PARAGRAPH
       unless @line ;
 
-    $static = 1 if $ret_type =~ s/^static\s+//;
+    $externC = 1 if $ret_type =~ s/^extern "C"\s+//;
+    $static  = 1 if $ret_type =~ s/^static\s+//;
 
     $func_header = shift(@line);
     blurt ("Error: Cannot parse function definition from '$func_header'"), next PARAGRAPH
@@ -536,8 +556,11 @@ EOF
 
     $xsreturn = 1 if $EXPLICIT_RETURN;
 
+    $externC = $externC ? qq[extern "C"] : "";
+
     # print function header
     print Q(<<"EOF");
+#$externC
 #XS(XS_${Full_func_name}); /* prototype to pass -Wmissing-prototypes */
 #XS(XS_${Full_func_name})
 #[[
@@ -1023,7 +1046,7 @@ sub print_section {
     for (;  defined($_) && !/^$BLOCK_re/o;  $_ = shift(@line)) {
 	print "$_\n";
     }
-    print "$ExtUtils::ParseXS::CountLines::SECTION_END_MARKER\n" if $WantLineNumbers;
+    print 'ExtUtils::ParseXS::CountLines'->end_marker, "\n" if $WantLineNumbers;
 }
 
 sub merge_section {
@@ -1812,6 +1835,7 @@ use vars qw($SECTION_END_MARKER);
 
 sub TIEHANDLE {
   my ($class, $cfile, $fh) = @_;
+  $cfile =~ s/\\/\\\\/g;
   $SECTION_END_MARKER = qq{#line --- "$cfile"};
   
   return bless {buffer => '',
@@ -1849,7 +1873,9 @@ sub UNTIE {
   # This sub does nothing, but is neccessary for references to be released.
 }
 
-
+sub end_marker {
+  return $SECTION_END_MARKER;
+}
 
 
 1;
